/*
 *
 *      Copyright (c) 2018-2025, frame All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: frame
 *
 */

package com.dragon.tj.portal.component.log.aspect;

import com.alibaba.fastjson.JSON;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.service.TokenService;
import com.dragon.tj.portal.component.log.annotation.SysLog;
import com.dragon.tj.portal.component.log.dto.SysLogDTO;
import com.dragon.tj.portal.component.log.event.SysLogEvent;
import com.dragon.tj.portal.component.log.uitl.LogTypeEnum;
import com.dragon.tj.portal.component.log.uitl.SysLogUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.util.Map;
import java.util.Objects;

/**
 * @author huey China.
 * @Description : 操作日志使用spring event异步入库
 * @Date Created in 2023/6/12 15:23
 */
@Component
@Slf4j
@Aspect
public class SysLogAspect {

    private static final String GET_METHOD = "GET";
    private static final String POST_METHOD = "POST";

    private static final Class<RequestBody> REQUEST_BODY_CLASS = RequestBody.class;

    @Autowired
    private ApplicationEventPublisher publisher;

    @Autowired
    private TokenService tokenService;

    @SneakyThrows
    @Around("@annotation(sysLog)")
    public Object around(ProceedingJoinPoint point, SysLog sysLog) {
        String strClassName = point.getTarget().getClass().getName();
        String strMethodName = point.getSignature().getName();
        log.debug("[类名]:{},[方法]:{}", strClassName, strMethodName);

        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.currentRequestAttributes()).getRequest();
        LoginUser loginUser = tokenService.getLoginUser(request);
        String createBy = "admin";
        String createUser = "admin";
        if (Objects.nonNull(loginUser)) {
            log.info("class-method-current-login {}-{}-{}", strClassName, strMethodName, JSON.toJSONString(loginUser));
            createBy = loginUser.getIdCard();
            createUser = loginUser.getUsername();
        }
        SysLogDTO logDTO = SysLogUtils.getSysLog();
        logDTO.setTitle(sysLog.value());
        logDTO.setModuleType(sysLog.module().getCode());
        logDTO.setModuleName(sysLog.module().getDesc());
        logDTO.setCreateBy(createBy);
        logDTO.setCreateUser(createUser);
        logDTO.setParams(this.params(request, point));
        // 发送异步日志事件
        Long startTime = System.currentTimeMillis();
        Object obj;
        try {
            obj = point.proceed();
        } catch (Exception e) {
            logDTO.setType(LogTypeEnum.ERROR.getType());
            logDTO.setException(e.getMessage());
            throw e;
        } finally {
            Long endTime = System.currentTimeMillis();
            logDTO.setTime(endTime - startTime);
            publisher.publishEvent(new SysLogEvent(logDTO));
        }
        return obj;
    }


    private String params(HttpServletRequest request1, ProceedingJoinPoint point) throws IOException {
        // 获取GET请求的参数列表及对应的值

        StringBuilder params = new StringBuilder();
        Map<String, String[]> getParameters = request1.getParameterMap();
        for (Map.Entry<String, String[]> entry : getParameters.entrySet()) {
            String parameterName = entry.getKey();
            String[] parameterValues = entry.getValue();
            // 处理GET请求的参数及对应的值
            params.append("GET-PARAM:" + parameterName + " - " + String.join(", ", parameterValues));
        }

        // 获取POST请求的参数列表及对应的值
        // 获取POST请求中的JSON值
        if (POST_METHOD.equalsIgnoreCase(request1.getMethod())) {
            // 获取JSON参数
            MethodSignature methodSignature = (MethodSignature) point.getSignature();

            int index = getIndexOfRequestBodyParameter(methodSignature);
            if (index > -1) {
                Object[] args = point.getArgs();
                // 若有参数，则参数
                Object arg = args[index];
                String s = JSON.toJSONString(arg);
                if (Objects.nonNull(s)) {
                    params.append("POST-PARAM:" + s);
                }
            }
        }
        return params.toString();
    }

    /**
     * 获取@RequestBody注解参数的索引
     */
    private int getIndexOfRequestBodyParameter(final MethodSignature methodSignature) {
        int index = -1;
        // 获取参数上的所有注解
        final Annotation[][] parameterAnnotations = methodSignature.getMethod().getParameterAnnotations();
        for (int i = 0; i < parameterAnnotations.length; i++) {
            Annotation[] annotations = parameterAnnotations[i];
            for (int j = 0; j < annotations.length; j++) {
                Annotation annotation = annotations[j];
                if (REQUEST_BODY_CLASS.equals(annotation.annotationType())) {
                    return i;
                }
            }
        }
        return index;
    }
}