package com.dragon.tj.portal.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dc.eventpoi.ExcelHelper;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.entity.AppInfo;
import com.dragon.tj.portal.entity.InstallInfo;
import com.dragon.tj.portal.entity.PageParam;
import com.dragon.tj.portal.entity.SysDictItem;
import com.dragon.tj.portal.mapper.app.AppInfoMapper;
import com.dragon.tj.portal.mapper.app.InstallInfoMapper;
import com.dragon.tj.portal.service.AppService;
import com.dragon.tj.portal.service.FileManageService;
import com.dragon.tj.portal.service.SysDictItemService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class AppServiceImpl implements AppService {
    private static final Logger LOGGER = LoggerFactory.getLogger(AppServiceImpl.class);

    @Autowired
    private AppInfoMapper appInfoMapper;

    @Autowired
    private InstallInfoMapper installInfoMapper;

    @Autowired
    private SysDictItemService sysDictItemService;

    @Autowired
    private FileManageService fileManageService;

    private static List<SysDictItem> appTypeDict;

    public List<SysDictItem> getAppTypeDict() {
        if (appTypeDict == null || appTypeDict.size() == 0) {
            appTypeDict = sysDictItemService.list(Wrappers.<SysDictItem>query().lambda().eq(SysDictItem::getType, "app_type").orderByAsc(SysDictItem::getSort));
        }

        return appTypeDict;
    }

    /*
     * CRUD
     * */
    @Override
    public int add(AppInfo appInfo, LoginUser loginUser) {
        appInfo.setCreateUser(loginUser.getUsername());

        return appInfoMapper.insert(appInfo);
    }

    @Override
    public int delete(Long id) {
        AppInfo appInfo = new AppInfo();
        appInfo.setId(id);
        appInfo.setDelFlag(1);

        return appInfoMapper.updateById(appInfo);
    }

    @Override
    public int update(AppInfo appInfo) {
        return appInfoMapper.updateById(appInfo);
    }

    @Override
    public AppInfo detail(Long id) {
        AppInfo appInfo = appInfoMapper.selectById(id);

        List<SysDictItem> appTypeList = getAppTypeDict();
        SysDictItem specialToolDict = appTypeList.stream().filter(item -> "专项工具".equals(item.getLabel())).findFirst().get();
        if (appInfo.getAppType() == specialToolDict.getId()){
            appInfo.setIsSpecial(1);
        }else{
            appInfo.setIsSpecial(0);
        }

        return appInfo;
    }

    /*
     * 获取类型
     * */
    @Override
    public R getType() {
        return R.ok(getAppTypeDict());
    }

    /*
     * 搜索App
     * */
    @Override
    public Page<AppInfo> search(PageParam<AppInfo> appInfoPage) {

        AppInfo appInfo = appInfoPage.getParams();
        Page<AppInfo> rowPage = new Page(appInfoPage.getPage(), appInfoPage.getSize());

        LambdaQueryWrapper<AppInfo> queryWrapper = new LambdaQueryWrapper<>();

        //应用名称
        queryWrapper.like(StringUtils.isNotEmpty(appInfo.getSystemName()), AppInfo::getSystemName, appInfo.getSystemName());
        //应用事权单位名称
        queryWrapper.like(StringUtils.isNotEmpty(appInfo.getDeptName()), AppInfo::getDeptName, appInfo.getDeptName());
        //应用类型
        if (appInfo.getIsSpecial() != null) {
            List<SysDictItem> appTypeDict = getAppTypeDict();
            SysDictItem specialToolDict = appTypeDict.stream().filter(item -> "专项工具".equals(item.getLabel())).findFirst().get();
            //专项工具
            queryWrapper.eq(appInfo.getIsSpecial() == 1, AppInfo::getAppType, specialToolDict.getId());
            //非专项
            queryWrapper.ne(appInfo.getIsSpecial() == 0, AppInfo::getAppType, specialToolDict.getId());
        }
        queryWrapper.eq(appInfo.getAppType() != null, AppInfo::getAppType, appInfo.getAppType());
        queryWrapper.eq(appInfo.getDelFlag() != null, AppInfo::getDelFlag, appInfo.getDelFlag());

        rowPage.addOrder(OrderItem.desc(appInfoPage.getOrder()));

        return appInfoMapper.selectPage(rowPage, queryWrapper);
    }


    /*
     * 安装应用
     * */
    @Override
    public int install(Integer appId, String userIdcard) {

        //是否已安装
        QueryWrapper<InstallInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", appId);
        wrapper.eq("user_idcard", userIdcard);
        InstallInfo installed = installInfoMapper.selectOne(wrapper);

        if (installed == null) {
            InstallInfo installInfo = new InstallInfo();
            installInfo.setAppId(appId);
            installInfo.setUserIdcard(userIdcard);

            return installInfoMapper.insert(installInfo);
        }

        installed.setDelFlag(0);
        installed.setUpdateTime(LocalDateTime.now());

        return installInfoMapper.updateById(installed);
    }

    /*
     * 卸载
     * */
    @Override
    public int uninstall(Integer appId, String idCard) {
        InstallInfo installInfo = new InstallInfo();
        installInfo.setDelFlag(1);
        installInfo.setUpdateTime(LocalDateTime.now());
        QueryWrapper<InstallInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("app_id", appId);
        wrapper.eq("user_idcard", idCard);

        return installInfoMapper.update(installInfo, wrapper);
    }

    /*
     *  已安装应用
     * */
    @Override
    public IPage<AppInfo> installApp(PageParam<AppInfo> page, LoginUser loginUser) {
        AppInfo params = page.getParams();
        IPage<AppInfo> iPage = new Page<>(page.getPage(), page.getSize());

        Map map = new HashMap();

        map.put("delFlag", params.getDelFlag());
        Integer appType = params.getAppType();
        if (appType != null) {
            List<SysDictItem> appTypeList = getAppTypeDict();
            SysDictItem specialToolDict = appTypeList.stream().filter(item -> "专项工具".equals(item.getLabel())).findFirst().get();
            map.put("std", specialToolDict.getId());//专项工具

            if (specialToolDict.getId().compareTo(appType) == 0) {
                map.put("isSt", 1);//专项工具查询
            }
        }

        map.put("idcard", loginUser.getIdCard());

        return appInfoMapper.getInstalledApp(iPage, map);

    }

    /*
     *  应用统计
     * */
    @Override
    public void updateAppStat() {
        //获取app安装次数
        List<Map<String, Long>> apps = installInfoMapper.getAppInstallCount();
        //更新star
        apps.forEach(app -> {
            AppInfo appInfo = new AppInfo();
            appInfo.setId(app.get("appId"));
            appInfo.setStar(app.get("count"));
            appInfoMapper.updateById(appInfo);
        });

    }

    @Override
    public int clickIncr(Long id) {
        QueryWrapper<AppInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("id", id);
        AppInfo appInfo = appInfoMapper.selectOne(wrapper);
        appInfo.setId(id);
        appInfo.setHits(appInfo.getHits() + 1);

        return appInfoMapper.updateById(appInfo);
    }

    @Override
    public boolean batchAdd(MultipartFile file) {
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
//            String path = AppService.class.getClassLoader().getResource("").getPath();
//            path = URLDecoder.decode(path, "UTF-8");
//            FileInputStream tempFis = new FileInputStream(new File(path + "file/appInfoTemplete.xlsx"));

            ClassPathResource classPathResource = new ClassPathResource("file/appInfoTemplete.xlsx");
            InputStream tempFis = classPathResource.getInputStream();

            List<AppInfo> appInfos = ExcelHelper.parseExcelToObject(
                    new BufferedInputStream(tempFis),
                    inputStream,
                    AppInfo.class, true);

            appInfos.forEach(appInfo -> {
                AppInfo appInfo1 = parseAppInfo(appInfo);
                this.add(appInfo1, SecurityUtils.getLoginUser());
            });

        } catch (Exception e) {
            LOGGER.error("上传出错", e);
            return false;
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return true;
    }

    @Override
    public Map<String, List<Long>> checkAppTypeById(List<Long> appIds) {

        List<SysDictItem> appTypeList = getAppTypeDict();
        SysDictItem specialToolDict = appTypeList.stream().filter(item -> "专项工具".equals(item.getLabel())).findFirst().get();

        List<AppInfo> appInfos = appInfoMapper.selectBatchIds(appIds);
        Map<String, List<Long>> appTYpe = new HashMap<>();

        List<Long> specialApp = appInfos.stream()
                .filter(t -> t.getAppType() == specialToolDict.getId())
                .map(AppInfo::getId)
                .collect(Collectors.toList());
        List<Long> otherApp = appInfos.stream()
                .filter(t -> t.getAppType() != specialToolDict.getId())
                .map(AppInfo::getId)
                .collect(Collectors.toList());

        appTYpe.put("special",specialApp);
        appTYpe.put("other",otherApp);

        return appTYpe;
    }

    private AppInfo parseAppInfo(AppInfo appInfo) {
        List<SysDictItem> appTypeDict = getAppTypeDict();

        //类型表码转换
        String appTypeName = appInfo.getTypeName();
        SysDictItem sysDictItem = appTypeDict.stream()
                .filter(t -> t.getLabel().equals(appTypeName))
                .findFirst().get();
        appInfo.setAppType(sysDictItem.getId());

        //在用标识转换
        appInfo.setActiveFlag("在用".equals(appInfo.getActiveName()) ? 1 : 0);

        //图标处理
        byte[] iconImageByte = appInfo.getImage();
        if (iconImageByte.length > 0) {

            appInfo.setIcon(fileManageService.upload(
                    iconImageByte,
                    appInfo.getSystemName() + ".jpg",
                    appInfo.getAppType()));
        }

        return appInfo;

    }

}
